require 'json'

# == Schema Information
#
# Table name: benchmark_batches
#
#  id               :integer          not null, primary key
#  performed_at     :datetime
#  created_at       :datetime
#  updated_at       :datetime
#  configuration_id :integer
#

class BenchmarkBatch < ActiveRecord::Base

  has_many :benchmark_results, dependent: :destroy, inverse_of: :benchmark_batch
  belongs_to :benchmark_configuration, inverse_of: :benchmark_batches
  has_and_belongs_to_many :tags

  validates_presence_of :performed_at

  def result_for(info)
    benchmark_results.where(benchmark_info: info).first
  end

  def all_benchmark_infos
    benchmark_results.map(&:benchmark_info)
  end

  def BenchmarkBatch.archive_directory
    archive_dir = File.join(Rails.root, 'public', 'archive')
    if (not File.exist? archive_dir) then
      Dir.mkdir(archive_dir)
    end
    return archive_dir
  end

  # Returns full path to archive file
  def archive_path
    return File.join(BenchmarkBatch.archive_directory, self.archive_filename)
  end

  def to_json()
    return JSON.parse( IO.read( self.archive_path ) )
  end

end
