require 'json'

class ImportQuery
  include ActiveModel::Model

  def store!


    data = data_from_json

    archive_name = "#{data["timestamp"]}_#{(data["tags"] || data["name"] || 'default').split(",").join('-')}_#{data["configuration"]["language"]}_#{data["configuration"]["os"]}_#{data["configuration"]["runtime"]}.json"
    archive_name = archive_name.tr(' :/', '___')

    # Step 2: store data to database for faster access
    # ------------------------------------------------
    batch = BenchmarkBatch.create!(
      benchmark_configuration: find_configuration(data),
      performed_at: DateTime.parse(data["timestamp"]),
      archive_filename: archive_name)
    data["outcomes"].each do |results|

      bench_info = BenchmarkInfo.where(name: results["benchmark"]["name"]).first
      bench_info = BenchmarkInfo.create!(benchmark_params(results)) unless bench_info

      result = BenchmarkResult.create!(
        benchmark_info_id: bench_info.id,
        benchmark_batch_id: batch.id)
      results["times"].each do |duration|
        result.benchmark_durations.create!(duration: duration)
      end

      results["parameters"].each do | name, value|
        result.benchmark_parameters.create!(name: name, value: value)
      end

    end

    tags = find_tags(data["tags"] || data["name"] || 'default')
    tags.each { |t| batch.tags << t }

    File.open( batch.archive_path, 'w') { |file| file.write(JSON.pretty_generate(data))  }


    # Done
    batch
  end

  private

  def benchmark_params(results)
    {
      name:               results["benchmark"]["name"],
      benchmark_class:    results["benchmark"]["class"],
      benchmark_selector: results["benchmark"]["selector"],
      description:        results["benchmark"]["description"]
    }
  end

  def performed_at_ensured
    if performed_at.present?
      performed_at
    else
      Date.today
    end
  end

  def find_configuration(data)
    lang = find_or_create_by_name(Language, data["configuration"]["language"])
    runtime = find_or_create_by_name(lang.runtimes, data["configuration"]["runtime"])
    os = find_or_create_by_name(OperatingSystem, data["configuration"]["os"])
    desc = data["configuration"]["description"] || ""

    configuration_params = {language: lang, runtime: runtime, operating_system: os, description: desc }
    configuration = BenchmarkConfiguration.where(configuration_params).first
    configuration = BenchmarkConfiguration.create!(configuration_params) unless configuration
    configuration
  end

  def find_or_create_by_name(ctx, name)
    inst = ctx.where(name: name).first
    inst = ctx.create!(name: name) unless inst
    inst
  end

  def find_tags(tagOrTags)
    tagOrTags.split(",").map do |tag|
      Tag.find_by_name(tag) || Tag.create!(name: tag)
    end
  end

end
